unit ISF_Utilities;

////////////////////////////////////////////////////////////////////////////////
// ISTVAN-STANDARD-FORMATTING AND MARSCHALL-CASCADE UTILITIES FOR DELPHI      //
// Version 1.0.0.0                                                            //
// by Daniel Marschall                                                        //
// Portions originally written in C by Benjamin Despres                       //
// Component of ViaThinkSoft Lychrel Solve Utility                            //
////////////////////////////////////////////////////////////////////////////////

interface

uses
  SysUtils,
  Classes,
  StrUtils;

type
  T_ISF_STATUS = (isf_ok, isf_mod9_error, isf_crc16_error, isf_digit_error, isf_not_exists);

function file_isf_crc16(const AFilename: string): word;
function string_isf_crc16(const AInput: string): word;
function file_isf_mod9_passed(AFilename: string): boolean;

function load_isf(const AFileName: string; var AInitialValue, AIterationCounter, ACurrentNumber: string): T_ISF_Status;
function save_isf(const AFileName, AInitialValue, AIterationCounter, ACurrentNumber: string): boolean;

type
  T_CAS_STATUS = (cas_ok, cas_crc16_error, cas_invalid_character, cas_digit_error, cas_initialvalue_error, cas_not_exists);

function cas_load(const AFileName: string; var ACas: string): T_CAS_STATUS;
procedure cas_save(filename, InitialZahl, IterationCounter, cas: string);

procedure cas_extend(var cas: string; const value: string; const position: integer);

implementation

// Translated from C from http://users.tmok.com/~pla/lychrel/isft2757.zip
// Thanks to Muetze1 for the translation of the functions crc16 and file_isf_crc16
// http://www.delphipraxis.net/post872465.html#872465

function crc16(const AData: Char; APoly, AResult: word): word;
// Appends one byte to a runing CRC-16 calculation
var
  i: integer;
begin
  AResult := AResult xor word(word(AData) shl 8);

  for i := 1 to 8 do
  begin
    if ( AResult and $8000 ) <> 0 then
      AResult := (AResult shl 1) xor APoly
    else
      AResult := AResult shl 1;
  end;

  result := AResult;
end;

function file_isf_crc16(const AFilename: string): word;
// Calculates the CRC-16 of the ISF file
var
  lFile: TStream;  // finp
  lData: Char;     // crcd
  lCRC: Word;      // crcr
begin
  lCRC := $ffff;

  try
    lFile := TFileStream.Create(AFilename, fmOpenRead);
    try
      lData := '0';

      while ( lFile.Read(lData, 1) = 1 ) and ( lData >= ' ' ) do ;
      while ( lFile.Read(lData, 1) = 1 ) and ( lData >= ' ' ) do ;

      while ( lFile.Read(lData, 1) = 1 ) do
        lCRC := crc16(lData, $1021, lCRC);
    finally
      lFile.Free;
    end;
  except
  end;

  result := lCRC;
end;

const
  MOD9_PASSED = 0;
  MOD9_CANNOT_OCCUR_ON_ITERATION = 1;
  MOD9_CANNOT_RESULT_FROM_STARING_VALUE = 2;

function mod9_check(val1, val2, iter: integer): integer;
// Given two numbers, an iteration, and a MOD-9 checksum,
// verifies that the latter number can result from the
// former after the selected number of iterations.
var
  i, temp, flag: integer;
begin
  flag := MOD9_PASSED;
  if (val1 = 0) then
  begin
    if (val1 <> val2) then flag := MOD9_CANNOT_RESULT_FROM_STARING_VALUE;
  end
  else if ((val1 = 3) or (val1 = 6)) then
  begin
    temp := val1;
    i := 0;
    while i < (iter mod 2) do
    begin
      temp := (temp*2) mod 9;
      inc(i);
    end;
    if (temp <> val2) then flag := MOD9_CANNOT_OCCUR_ON_ITERATION;
    if ((val2 <> 6) and (val2 <> 3)) then flag := MOD9_CANNOT_RESULT_FROM_STARING_VALUE;
  end
  else
  begin
    temp := val1;
    i := 0;
    while i < (iter mod 6) do
    begin
      temp := (temp*2) mod 9;
      inc(i);
    end;
    if (temp <> val2) then flag := MOD9_CANNOT_OCCUR_ON_ITERATION;
    if ((val2 = 0) or (val2 = 3) or (val2 = 6)) then flag := MOD9_CANNOT_RESULT_FROM_STARING_VALUE;
  end;

  result := flag;
end;

// Self created functions for easier use

function string_isf_crc16(const AInput: string): word;
var
  lCRC: Word;
  i: integer;
begin
  lCRC := $ffff;

  // String beginnt mit Initial value!

  for i := 1 to length(AInput) do
  begin
    lCRC := crc16(AInput[i], $1021, lCRC);
  end;

  result := lCRC;
end;

function file_isf_mod9_passed(AFilename: string): boolean;
var
  txt: TextFile;
  tmp, bignum: string;
  i, val1, val2, iter: integer;
begin
  AssignFile(txt, AFileName);
  Reset(txt);

  ReadLn(txt, tmp);

  // VAL1
  val1 := 0;
  ReadLn(txt, tmp);
  tmp := copy(tmp, 19, length(tmp)-18);
  tmp := ReverseString(tmp);
  for i := 1 to length(tmp) do val1 := ((val1+strtoint(tmp[i])) mod 9);

  // ITER
  ReadLn(txt, tmp);
  iter := strtoint(copy(tmp, 19, length(tmp)-18));

  ReadLn(txt, tmp);

  // VAL2
  val2 := 0;
  bignum := '';
  while not eof(txt) do
  begin
    ReadLn(txt, tmp);
    bignum := bignum + tmp;
  end;
  CloseFile(txt);
  for i := 1 to length(bignum) do val2 := ((val2+strtoint(bignum[i])) mod 9);

  result := mod9_check(val1, val2, iter) = MOD9_PASSED;
end;

function load_isf(const AFileName: string; var AInitialValue, AIterationCounter, ACurrentNumber: string): T_ISF_Status;
var
  txt: TextFile;
  tmp: string;
  checksum: word;
  digit_check: integer;
begin
  if fileexists(AFileName) then
  begin
    if not file_isf_mod9_passed(AFileName) then
    begin
      result := isf_mod9_error;
      exit;
    end;

    checksum := file_isf_crc16(AFileName);

    AssignFile(txt, AFileName);
    Reset(txt);

    ReadLn(txt, tmp);
    if checksum <> strtoint(copy(tmp, 17, length(tmp)-16)) then
    begin
      result := isf_crc16_error;
      exit;
    end;

    ReadLn(txt, tmp);
    AInitialValue := copy(tmp, 19, length(tmp)-18);

    ReadLn(txt, tmp);
    AIterationCounter := copy(tmp, 19, length(tmp)-18);

    ReadLn(txt, tmp);
    digit_check := strtoint(copy(tmp, 19, length(tmp)-18));

    ACurrentNumber := '';
    while not eof(txt) do
    begin
      ReadLn(txt, tmp);
      ACurrentNumber := ACurrentNumber + tmp;
    end;

    if digit_check <> length(ACurrentNumber) then
    begin
      result := isf_digit_error;
      exit;
    end;

    CloseFile(txt);

    result := isf_ok;
  end
  else
  begin
    result := isf_not_exists;
  end;
end;

function save_isf(const AFileName, AInitialValue, AIterationCounter, ACurrentNumber: string): boolean;
var
  txt: TextFile;
  i: integer;
  daten: string;
begin
  result := true;
  try
    AssignFile(txt, AFileName);
    Rewrite(txt);

    daten := 'Initial value:    ' + AInitialValue+#13#10 +
             'Iteration:        ' + AIterationCounter + #13#10 +
             'Number of digits: ' + inttostr(length(ACurrentNumber)) + #13#10;

    for i := 0 to length(ACurrentNumber) div 70 do
    begin
      daten := daten + copy(ACurrentNumber, 70*i+1, 70) + #13#10;
    end;

    Write(txt, 'Automatic save #' + inttostr(string_isf_crc16(daten)) + #13#10);
    Write(txt, daten);

    CloseFile(txt);
  except
    result := false;
  end;
end;

// CASCADE-UTILITIES

const
  cas_code = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz*';

function Cas_Checksum(input: string): string;
var
  i, va, error, quersumme: Integer;
  str: string;
begin
  quersumme := 0;
  for i := 1 to 7 do
  begin
    str := copy(input, i, 1);
    val(str, va, error);
    quersumme := quersumme + va;
  end;
  result := cas_code[quersumme];
end;

function cas_load(const AFileName: string; var ACas: string): T_CAS_STATUS;
var
  checksum: word;
  txt: TextFile;
  tmp: string;
  chk_iv, chk_it: string;
  i: Integer;
begin
  aCAS := '';

  if fileexists(AFileName) then
  begin
    checksum := file_isf_crc16(AFileName);

    AssignFile(txt, AFileName);
    Reset(txt);

    ReadLn(txt, tmp);
    if checksum <> strtoint(copy(tmp, 17, length(tmp)-16)) then
    begin
      result := cas_crc16_error;
      exit;
    end;

    ReadLn(txt, tmp);
    chk_iv := copy(tmp, 19, length(tmp)-18);

    ReadLn(txt, tmp);
    chk_it := copy(tmp, 19, length(tmp)-18);

    ReadLn(txt, tmp);

    while not eof(txt) do
    begin
      ReadLn(txt, tmp);
      aCAS := aCAS + tmp;
    end;

    for i := 1 to length(aCas) do
    begin
      if AnsiPos(aCas[i], cas_code) = 0 then
      begin
        result := cas_invalid_character;
        exit;
      end;
    end;

    if copy(aCas, 1, 1) <> Cas_Checksum(chk_iv) then
    begin
      result := cas_initialvalue_error;
      exit;
    end;

    if length(aCas) <> strtoint(chk_it)+1 then
    begin
      result := cas_digit_error;
      exit;
    end;

    CloseFile(txt);

    result := cas_ok;
  end
  else
  begin
    result := cas_not_exists;
  end;
end;

procedure cas_save(filename, InitialZahl, IterationCounter, cas: string);
var
  txt: TextFile;
  daten: string;
  i: integer;
begin
  AssignFile(txt, ExtractFilePath(ParamStr(0))+'Data\'+filename+'.cas');
  Rewrite(txt);

  daten := 'Initial value:    ' + InitialZahl + #13#10 +
           'Iteration:        ' + IterationCounter + #13#10 + #13#10;

  for i := 0 to length(cas) div 70 do
  begin
    daten := daten + copy(cas, 70*i+1, 70) + #13#10;
  end;

  Write(txt, 'Automatic save #' + inttostr(string_isf_crc16(daten)) + #13#10);
  Write(txt, daten);

  CloseFile(txt);
end;

procedure StringSetElement(var input: string; const element: integer; const value: char);
begin
  while length(input) < element do input := input + '?';
  input[element] := value;
end;

procedure cas_extend(var cas: string; const value: string; const position: integer);
var
  hash_char: string;
begin
  hash_char := cas_checksum(value);
  StringSetElement(CAS, position, hash_char[1]);
end;

end.
