unit LychrelThread;

interface

uses
  Windows, SysUtils, Classes;

type
   TLychrelStatusCallback = procedure(digitcount, iterations, speed: string) of object;
   TSolvedCallback = procedure(ainitialzahl, adigits, aiterationcounter: string) of object;

   TLychrelSolve = class(TThread)
     private
        cas: string;
        aktuell: string;
        initialzahl: string;
        iterationcounter: string;
        conf_saveinterval: integer;
        solved: boolean;
        procedure AfterTermination(Sender: TObject);
        procedure Speichern(filename: string);
     protected
        procedure Execute; override;
     public
        StatusCallback: TLychrelStatusCallback;
        SolvedCallback: TSolvedCallback;
        constructor create(ainitialzahl, aaktuell, aiterationcounter, acas: string; aconf_saveinterval: integer); virtual;
   end;

implementation

uses
  UBigIntsV3, ISF_Utilities;

// http://www.delphipraxis.net/topic40702.html

procedure StrReverseInPlace(var S: AnsiString);
var
  P1, P2: PChar;
  C: AnsiChar;
begin
  UniqueString(S);
  P1 := PChar(S);
  P2 := P1 + SizeOf(AnsiChar) * (Length(S) - 1);
  while P1 < P2 do
  begin
    C := P1^;
    P1^ := P2^;
    P2^ := C;
    Inc(P1);
    Dec(P2);
  end;
end;

function StrReverse(const S: AnsiString): AnsiString;
begin
  Result := S;
  StrReverseInplace(Result);
end;

procedure TLychrelSolve.Speichern(filename: string);
begin
  save_isf(ExtractFilePath(ParamStr(0))+'Data\'+filename+'.isf', initialzahl, iterationcounter, aktuell);
end;

function add_separators(input: string): string;
var
  i1: tinteger;
begin
  i1 := TInteger.create;
  try
    i1.Assign(input);
    result := i1.ConvertToDecimalString(true);
  finally
    i1.Free;
  end;
end;

constructor TLychrelSolve.create(ainitialzahl, aaktuell, aiterationcounter, acas: string; aconf_saveinterval: integer);
begin
  inherited create(true);

  initialzahl := ainitialzahl;
  aktuell := aaktuell;
  iterationcounter := aiterationcounter;
  cas := acas;

  conf_saveinterval := aconf_saveinterval;

  OnTerminate := AfterTermination;
end;

procedure TLychrelSolve.Execute;
var
  i1, i2: tinteger;
  Inversion: string;
  tick: dword;
  iter: string;
  digi: integer;
  stat_tick_diff, stat_digi_diff: integer;
  stat_iter_diff: int64;
  last_save: integer;
  sdigitcount, siterations, sspeed: string;
begin
  i1 := TInteger.create;
  i2 := TInteger.create;

  solved := false;
  tick := GetTickCount;
  iter := iterationcounter;
  digi := length(aktuell);
  last_save := digi; // fr den Fall Neustart bei z.B. 10.000

  try
    repeat
      // Hash-Cascade
      cas_extend(cas, aktuell, StrToInt(iterationcounter)+1);

      if Terminated then break;

      // Zwischenspeicherung
      if (length(aktuell) mod conf_saveinterval = 0) and (last_save <> length(aktuell)) then
      begin
        Speichern(initialzahl+'_'+iterationcounter+'_'+IntToStr(length(aktuell)));
        copyfile(
          PChar(ExtractFilePath(ParamStr(0))+'Data\'+
            initialzahl+'_'+iterationcounter+'_'+IntToStr(length(aktuell))+'.isf'),
          Pchar(ExtractFilePath(ParamStr(0))+'Data\'+
            initialzahl+'_master.isf'),
          False);

        cas_save(initialzahl+'_'+iterationcounter+'_'+IntToStr(length(aktuell)), InitialZahl, IterationCounter, cas);

        copyfile(
          PChar(ExtractFilePath(ParamStr(0))+'Data\'+
            initialzahl+'_'+iterationcounter+'_'+IntToStr(length(aktuell))+'.cas'),
          Pchar(ExtractFilePath(ParamStr(0))+'Data\'+
            initialzahl+'_master.cas'),
          False);

        last_save := length(aktuell);
      end;

      // 1. Inversion finden
      Inversion := StrReverse(aktuell);

      // 2. Bereits ein Palindrom?
      if (aktuell = Inversion) and (iterationcounter <> '0') then
      begin
        Solved := true;
        break;
      end;

      // 3. Iterationcounter erhhen
      i1.Assign(iterationcounter);
      i1.Add(1);
      iterationcounter := i1.ConvertToDecimalString(false);

      // 4. Addieren
      i1.assign(aktuell);
      i2.assign(inversion);
      i1.add(i2);
      aktuell := i1.ConvertToDecimalString(false);

      // Geschwindigkeit messen
      stat_tick_diff := GetTickCount() - tick;

      if stat_tick_diff >= 1000 then
      begin
        if assigned(StatusCallback) then
        begin
          i1.Assign(iterationcounter);
          i2.Assign(iter);
          i1.Subtract(i2);
          i1.ConvertToInt64(stat_iter_diff);

          stat_digi_diff := length(aktuell) - digi;

          sspeed := FloatToStrF((stat_iter_diff / stat_tick_diff * 1000) * (stat_digi_diff / stat_tick_diff * 1000), ffNumber, 10, 2) + ' LP/s' + #13#10 +
                    FloatToStrF((stat_iter_diff / stat_tick_diff * 1000), ffNumber, 10, 2) + ' It/s' + #13#10 +
                    FloatToStrF((stat_digi_diff / stat_tick_diff * 1000), ffNumber, 10, 2) + ' DI/s';
        end;

        iter := iterationcounter;
        digi := length(aktuell);
        tick := GetTickCount;
      end;

      if assigned(StatusCallback) then
      begin
        siterations := add_separators(iterationcounter);
        sdigitcount := FloatToStrF(length(aktuell), ffNumber, 10, 0);
        StatusCallback(sdigitcount, siterations, sspeed);
      end;
    until false;
  finally
    i1.free;
    i2.free;
  end;
end;

procedure TLychrelSolve.AfterTermination(Sender: TObject);
begin
  // Ergebnisse und Konfiguration abspeichern

  Speichern(initialzahl+'_master');
  cas_save(initialzahl+'_master', InitialZahl, IterationCounter, cas);

  if Solved and assigned(SolvedCallback) then
  begin
    SolvedCallback(add_separators(initialzahl),
                   FloatToStrF(length(aktuell), ffNumber, 10, 0),
                   add_separators(iterationcounter));
  end;
end;

end.

